# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


class TaskPred:

    obj_list = []

    def __init__(self, params):
        self.task_pred_id = params.get('task_pred_id').strip() if params.get('task_pred_id') else None
        self.task_id = int(params.get('task_id')) if params.get('task_id') else None
        self.pred_task_id = int(params.get('pred_task_id')) if params.get('pred_task_id') else None
        self.proj_id = int(params.get('proj_id').strip()) if params.get('proj_id') else None
        self.pred_proj_id = int(params.get('proj_id').strip()) if params.get('pred_proj_id') else None
        self.pred_type = params.get('pred_type').strip() if params.get('pred_type') else None
        self.lag_hr_cnt = int(params.get('lag_hr_cnt').strip()) if params.get('lag_hr_cnt') else None
        self.float_path = params.get('float_path').strip() if params.get('float_path') else None
        self.aref = params.get('aref').strip() if params.get('aref') else None
        self.arls = params.get('arls').strip() if params.get('arls') else None
        self.comments = params.get('comments').strip() if params.get('comments') else None
        TaskPred.obj_list.append(self)

    def get_id(self):
        return self.task_pred_id

    def get_tsv(self):
        tsv = ['%R', self.task_pred_id, self.task_id, self.pred_task_id, self.proj_id, self.pred_proj_id,
               self.pred_type, self.lag_hr_cnt, self.comments, self.float_path, self.aref, self.arls]
        return tsv
    def __repr__(self):
        return str(self.task_id) + '- ' + self.pred_type + ' ->' + str(self.pred_task_id) + ' lag: ' + str(self.lag_hr_cnt)


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


'''
This file starts the process of reading and parsing xer files

'''
import csv
import mmap
import codecs
from xerparser import *
from typing import List
from xerparser.model.classes.data import Data
from xerparser.write import writeXER

class Reader:

    """
    Reader class is the main parser for xer files. It takes a file path as a parameter.
    The read records are parsed into classes depending on their types like activities, wbs,etc.

    :param <file>: is an xer file that need to be parsed
    :return: Reader object that contains collection of parsed records
    :Example:

    from xerparser.reader import Reader
    file = Reader('file.xer')

    """

    current_table = ''
    current_headers = []

    def write(self, filename=None):
        if filename is None:
            raise Exception("You have to provide the filename")
        writeXER(self, filename)

    def create_object(self, object_type, params):
        """

        Args:
            object_type:
            params:

        Returns:

        """
        if object_type.strip() == "CURRTYPE":
            self._currencies.add(params)
        elif object_type.strip() == "ROLES":
            self._roles.add(params)
        elif object_type.strip() == "ACCOUNT":
            self._accounts.add(params)
        elif object_type.strip() == "ROLERATE":
            self._rolerates.add(params)
        elif object_type.strip() == "OBS":
            self._obss.add(params)
        elif object_type.strip() == "RCATTYPE":
            self._rcattypes.add(params)
        elif object_type.strip() == "UDFTYPE":
            self._udftypes.add(params)
        elif object_type.strip() == "RCATVAL":
            self._rcatvals.add(params)
        elif object_type.strip() == "PROJECT":
            self._projects.add(params, self._data)
        elif object_type.strip() == "CALENDAR":
            self._calendars.add(params)
        elif object_type.strip() == "SCHEDOPTIONS":
            self._schedoptions.add(params)
        elif object_type.strip() == "PROJWBS":
            self._wbss.add(params, self._data)
        elif object_type.strip() == "RSRC":
            self._resources.add(params)
        elif object_type.strip() == "RSRCCURVDATA":
            self._rsrcurves.add(params)
        elif object_type.strip() == "ACTVTYPE":
            self._acttypes.add(params)
        elif object_type.strip() == "PCATTYPE":
            self._pcattypes.add(params)
        elif object_type.strip() == "PROJPCAT":
            self._projpcats.add(params)
        elif object_type.strip() == "PCATVAL":
            self._pcatvals.add(params)
        elif object_type.strip() == "RSRCRATE":
            self._rsrcrates.add(params)
        elif object_type.strip() == "RSRCRCAT":
            self._rsrccats.add(params)
        elif object_type.strip() == "TASK":
            self._tasks.add(params, self._data)
        elif object_type.strip() == "ACTVCODE":
            self._actvcodes.add(params)
        elif object_type.strip() == "TASKPRED":
            self._predecessors.add(params)
        elif object_type.strip() == "TASKRSRC":
            self._activityresources.add(params, self._data)
        elif object_type.strip() == "TASKPROC":
            self._taskprocs.add(params)
        elif object_type.strip() == "TASKACTV":
            self._activitycodes.add(params, self._data)
        elif object_type.strip() == "UDFVALUE":
            self._udfvalues.add(params)
        elif object_type.strip() == "FINTMPL":
            self._fintmpls.add(params)
        elif object_type.strip() == "NONWORK":
            self._nonworks.add(params)

    def summary(self):
        print('Number of activities: ', self.tasks.count)
        print('Number of relationships: ', len(TaskPred.obj_list))

    @property
    def projects(self) -> List[Project]:
        """
        Projects

        :return: list of all projects contained in the xer file

        todo:: text

        """
        return self._projects

    @property
    def activities(self) -> List[Tasks]:
        """
        Property to retrieve list of tasks
        Returns: list of tasks
        """
        return self._tasks

    @property
    def wbss(self) -> WBSs:
        """
        Property to return all wbs elements in the parsed file
        Returns:
            list of wbs elements
        """
        return self._wbss

    @property
    def relations(self) -> Predecessors:
        """
        Property to retrieve relationships from parsed file
        Returns:
            list of relations
        """
        return self._predecessors

    @property
    def resources(self) -> Resources:
        """
        Property to return a list of resources from parsed file
        Returns:
            list of resources of type Resources
        """
        return self._resources

    @property
    def accounts(self) -> Accounts:
        """
        Property to return a list of accounts from parsed files
        Returns:
            list of accounts of type Accounts
        """
        return self._accounts

    @property
    def activitycodes(self) -> ActivityCodes:
        """
        Property to return a list of activity codes from parsed files
        Returns:
            list of accounts of type ActivityCode

        """
        return self._activitycodes
    @property
    def actvcodes(self) -> TaskActvs:
        return self._actvcodes

    @property
    def acttypes(self) -> ActTypes:
        return self._acttypes

    @property
    def calendars(self) -> Calendars:
        return self._calendars

    @property
    def currencies(self) -> Currencies:
        return self._currencies

    @property
    def obss(self) -> OBSs:
        return self._obss

    @property
    def rcattypes(self) -> RCatTypes:
        return self._rcattypes

    @property
    def rcatvals(self) -> RCatVals:
        return self._rcatvals

    @property
    def rolerates(self) -> RoleRates:
        return self._rolerates

    @property
    def roles(self) -> Roles:
        return self._roles

    @property
    def resourcecurves(self) -> ResourceCurves:
        return self._rsrcurves

    @property
    def resourcerates(self) -> ResourceRates:
        return self._rsrcrates

    @property
    def resourcecategories(self) -> ResourceCategories:
        return self._rsrccats

    @property
    def scheduleoptions(self) -> SchedOptions:
        return self._schedoptions

    @property
    def activityresources(self) -> ActivityResources:
        return self._activityresources

    @property
    def udfvalues(self) -> UDFValues:
        return self._udfvalues

    @property
    def udftypes(self) -> UDFTypes:
        return self._udftypes

    @property
    def pcattypes(self) -> List[PCatTypes]:
        return self._pcattypes

    @property
    def pcatvals(self) -> List[PCatVals]:
        return self._pcatvals

    @property
    def projpcats(self) -> List[ProjCat]:
        return self._projpcats

    @property
    def taskprocs(self) -> List[TaskProc]:
        return self._taskprocs

    @property
    def fintmpls(self) -> List[FinTmpl]:
        return self._fintmpls

    @property
    def nonworks(self) -> List[NonWork]:
        return self._nonworks

    def __init__(self, filename):
        file = open(filename, 'r')
        self.file = filename
        self._tasks = Tasks()
        self._predecessors = Predecessors()
        self._projects = Projects()
        self._wbss = WBSs()
        self._resources = Resources()
        self._accounts = Accounts()
        self._actvcodes = ActivityCodes()
        self._activitycodes = TaskActvs()
        self._acttypes = ActTypes()
        self._calendars = Calendars()
        self._currencies = Currencies()
        self._obss = OBSs()
        self._rcatvals = RCatVals()
        self._rcattypes = RCatTypes()
        self._rolerates = RoleRates()
        self._roles = Roles()
        self._rsrcurves = ResourceCurves()
        self._rsrcrates = ResourceRates()
        self._rsrccats = ResourceCategories()
        self._schedoptions = SchedOptions()
        self._activityresources = ActivityResources()
        self._udftypes = UDFTypes()
        self._udfvalues = UDFValues()
        self._pcattypes = PCatTypes()
        self._pcatvals = PCatVals()
        self._projpcats = ProjCats()
        self._taskprocs = TaskProcs()
        self._fintmpls = FinTmpls()
        self._nonworks = NonWorks()
        self._data = Data()
        self._data.projects = self._projects
        self._data.wbss = self._wbss
        self._data.tasks = self._tasks
        self._data.resources = self._resources
        self._data.taskresource = self._activityresources
        self._data.taskactvcodes = self._activitycodes
        with codecs.open(filename, encoding='utf-8', errors='ignore') as tsvfile:
            stream = csv.reader(tsvfile, delimiter='\t')
            for row in stream:
                if row[0] =="%T":
                    current_table = row[1]
                elif row[0] == "%F":
                    current_headers = [r.strip() for r in row[1:]]
                elif row[0] == "%R":
                    zipped_record = dict(zip(current_headers, row[1:]))
                    self.create_object(current_table, zipped_record)

        # for line in content:
        #     line_lst = line.split('\t')
        #     if line_lst[0] == "%T":
        #         current_table = line_lst[1]
        #     elif line_lst[0] == "%R":
        #         self.create_object(current_table, line_lst[1:])

    def get_num_lines(self, file_path):
        fp = open(file_path, "r+")
        buf = mmap.mmap(fp.fileno(), 0)
        lines = 0
        while buf.readline():
            lines += 1
        return lines



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


import re
import datetime


class CalendarData:
    working_days = dict()
    exceptions = []

    def __init__(self, text):
        self.text = text
        cal2 = []
        cal = re.findall("\(\d\|\|\d+\(\)", text)
        for c in cal:
            c2 = c.split("||")[1]
            cal2.append(c2.split("()")[0])
        val = re.split("\(\d\|\|\d+\(\)", text.strip())
        x = 0

        self.data = dict()
        i = 0
        for c in range(len(cal2)):
            if c >= 1:
                self.data[cal2[c]] = val[c + 1]

            else:
                self.data[cal2[c]] = ''
        self.exceptions = self.get_exceptions()
        self.working_days = self.get_days()

    def xldate_to_datetime(self, xldate):
        temp = datetime.datetime(1899, 12, 30)
        delta = datetime.timedelta(days=xldate)
        return temp + delta

    def get_work_pattern(self):
        pattern = r"\(\d\|\|\d\(\)\("
        dayName = {2: "Monday", 3: "Tuesday", 4: "Wednesday", 5: "Thursday", 6: "Friday", 7: "Saturday", 1: "Sunday"}
        tx = self.text.split("(0||VIEW(ShowTotal|Y)())")
        days = re.split(pattern, tx[0])
        dys = re.findall(pattern, self.text)
        lst_dow = []
        for day, d in zip(days[1:], dys):
            dow = int(d.replace("(0||", "").replace(")", "").replace("(", ""))
            starts = re.findall("s\|\d\d\:\d\d", day)
            finishes = re.findall("f\|\d\d\:\d\d", day)

            day_dict = {'DayOfWeek': dayName[dow], 'WorkTimes': [], 'ifc': None}
            for s, f in zip(starts, finishes):
                s_c = datetime.datetime.strptime(s.replace("s|", ""), "%H:%M").time()
                f_c = datetime.datetime.strptime(f.replace("f|", ""), "%H:%M").time()
                day_dict['WorkTimes'].append({"Start": s_c, "Finish": f_c})
            lst_dow.append(day_dict)
        return (lst_dow)

    def get_exceptions(self):
        base_datetime = datetime.datetime(1899, 12, 30)
        excep_dates = re.findall(r"\d{5,}", self.text)
        exceptions = []
        for exc in excep_dates:
            exc = int(exc)
            delta = datetime.timedelta(exc)
            exc_date = base_datetime + delta
            dt = datetime.datetime.fromtimestamp(exc, tz=datetime.timezone.utc)
            exceptions.append(exc_date)
        return exceptions

    def get_days(self):
        if not self.data:
            return
        first = re.findall("\(\d\|\|\d\(\)(.*?)\)\)", self.text)
        days = dict()
        i = 0
        for x in first:
            second = x.split("\x7f\x7f")
            x = x.replace("(", "").replace(")", "").replace(" ", "").strip()
            key = str(i + 1)
            if len(x) > 1:
                days[key] = (len(x) > 0)
            else:
                days[key] = False
            i += 1
        self.working_days = days
        return self.working_days




class Data:

    def __init__(self) -> None:
        self.projects = None
        self.wbss = None
        self.tasks = None
        self.resources = None
        self.taskresource = None
        self.taskactvcodes = None


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


class UDFValue:

    udf_code_id = None
    udf_type_id = None
    fk_id = None
    proj_id = None
    udf_number = None
    udf_text = None

    def __init__(self, params):

        self.udf_type_id = params.get('udf_type_id').strip() if params.get('udf_type_id') else None
        self.fk_id = params.get('fk_id').strip() if params.get('fk_id') else None
        self.proj_id = params.get('proj_id').strip() if params.get('proj_id') else None
        self.udf_date = params.get('udf_date').strip() if params.get('udf_date') else None
        self.udf_number = params.get('udf_number').strip() if params.get('udf_number') else None
        self.udf_text = params.get('udf_text').strip() if params.get('udf_text') else None
        self.udf_code_id = params.get('udf_code_id').strip() if params.get('udf_code_id') else None

    def get_id(self):
        return self.udf_type_id

    def get_tsv(self):
        tsv = ["%R", self.udf_type_id, self.fk_id, self.proj_id, self.udf_date, self.udf_number, self.udf_text,
               self.udf_code_id]
        return tsv

    @staticmethod
    def find_by_id(code_id, activity_code_dict):
        return {k: v for k, v in activity_code_dict.items() if v.actv_code_id == code_id}

    def __repr__(self):
        return self.udf_text + '->' + self.udf_code_id


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


class TaskActv:
    obj_list = []

    def __init__(self, params, data):
        self.task_id = int(params.get('task_id').strip()) if params.get('task_id') else None
        self.actv_code_type_id = params.get('actv_code_type_id').strip()
        self.actv_code_id = int(params.get('actv_code_id').strip()) if params.get('actv_code_id') else None
        self.proj_id = int(params.get('proj_id').strip()) if params.get('proj_id') else None
        self.data = data
        TaskActv.obj_list.append(self)

    def get_tsv(self):
        tsv = ['%R', self.task_id, self.actv_code_type_id, self.actv_code_id, self.proj_id]
        return tsv

    def get_id(self):
        return self.task_id

    @staticmethod
    def find_by_id(code_id, activity_code_dict):
        return {k: v for k, v in activity_code_dict.items() if v.actv_code_id == code_id}

    def __repr__(self):
        return str(self.task_id) + '->' + str(self.actv_code_id)


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.

import locale
class ActType:
    obj_list =[]

    def __init__(self, params):
        # Unique ID generated by the system.
        self.actv_code_type_id = int(params.get('actv_code_type_id').strip()) if params.get('actv_code_type_id') else None
        # The maximum number of characters allowed for values of this Activity Code.
        self.actv_short_len = locale.atof(params.get('actv_short_len').strip()) if params.get('actv_short_len') else None
        # Sequence number for sorting.
        self.seq_num = int(params.get('seq_num').strip()) if params.get('seq_num') else None
        # Each Activity Code has a list of possible values, any of which can be assigned to an activity. Activity
        # Codes allow you to classify and categorize activities.
        self.actv_code_type = params.get('actv_code_type').strip() if params.get('actv_code_type') else None
        # Identifies the project which owns this Activity Code (if this is a project-level Activity Code).
        self.proj_id = int(params.get('proj_id')) if params.get('proj_id') else None
        # EPS Id element that
        self.wbs_id = params.get('wbs_id').strip() if params.get('wbs_id') else None
        self.actv_code_type_scope = params.get('actv_code_type_scope').strip() if params.get('actv_code_type_scope') else None
        ActType.obj_list.append(self)

    def get_id(self):
        return self.actv_code_type_id

    def get_tsv(self):
        tsv = ['%R', self.actv_code_type_id, self.actv_short_len, self.seq_num,
               self.actv_code_type, self.proj_id, self.wbs_id, self.actv_code_type_scope]
        return tsv
    @classmethod
    def find_by_id(cls, id):
        """ Function to search list of activity code type by an ID

        Args:
            id: Unique ID generated by the system.

        Returns: ActType that matches the ID

        """
        obj = list(filter(lambda x: x.actv_code_type_id == id, cls.obj_list))
        if len(obj) > 0:
            return obj[0]
        else:
            obj = None
        return obj


    def __repr__(self):
        return self.actv_code_type




# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.account import Account

class Accounts:

    def __init__(self):
        self._accounts = []
        self.index = 0

    def add(self, params):
        self._accounts.append(Account(params))

    def get_tsv(self):
        if len(self._accounts) > 0:
            tsv = list()
            tsv.append(["%T", "ACCOUNT"])
            tsv.append(["%F", "acct_id", "parent_acct_id", "acct_seq_num", "acct_name", "acct_short_name",
                        "acct_descr"])
            for account in self._accounts:
                tsv.append(account.get_tsv())
            return tsv
        return []

    def count(self):
        return len(self._accounts)

    def __iter__(self):
        return self

    def __next__(self) -> Account:
        if self.index >= len(self._accounts):
            raise StopIteration
        idx = self.index
        self.index += 1
        return self._accounts[idx]


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.tasks import Tasks
from xerparser.model.classes.task import Task

class WBS:
    obj_list = []

    def __init__(self, params, data=None):
        self.wbs_id = int(params.get('wbs_id').strip()) if params.get('wbs_id') else None
        self.proj_id = int(params.get('proj_id').strip()) if params.get('proj_id') else None
        self.obs_id = params.get('obs_id').strip()
        self.seq_num = params.get('seq_num').strip()
        self.est_wt = params.get('est_wt')
        self.proj_node_flag = params.get('proj_node_flag').strip()
        self.sum_data_flag = params.get('sum_data_flag').strip()
        self.status_code = params.get('status_code').strip()
        self.wbs_short_name = params.get('wbs_short_name').strip()
        self.wbs_name = params.get('wbs_name').strip()
        self.phase_id = params.get('phase_id').strip()
        self.parent_wbs_id = int(params.get('parent_wbs_id')) if params.get('parent_wbs_id') else None
        self.ev_user_pct = params.get('ev_user_pct').strip()
        self.ev_etc_user_value = params.get('ev_etc_user_value').strip()
        self.orig_cost = params.get('orig_cost').strip()
        self.indep_remain_total_cost = params.get('indep_remain_total_cost').strip()
        self.ann_dscnt_rate_pct = params.get('ann_dscnt_rate_pct').strip()
        self.dscnt_period_type = params.get('dscnt_period_type').strip()
        self.indep_remain_work_qty = params.get('indep_remain_work_qty').strip()
        self.anticip_start_date = params.get('anticip_start_date').strip()
        self.anticip_end_date = params.get('anticip_end_date').strip()
        self.ev_compute_type = params.get('ev_compute_type').strip()
        self.ev_etc_compute_type = params.get('ev_etc_compute_type').strip()
        self.guid = params.get('guid').strip()
        self.tmpl_guid = params.get('tmpl_guid').strip()
        self.plan_open_state = params.get('plan_open_state').strip() if params.get('plan_open_state') else None
        self.data = data
        WBS.obj_list.append(self)

    def get_id(self):
        return self.wbs_id

    def get_tsv(self):
        tsv = ['%R', self.wbs_id, self.proj_id, self.obs_id, self.seq_num, self.est_wt,
               self.proj_node_flag, self.sum_data_flag, self.status_code, self.wbs_short_name,
               self.wbs_name, self.phase_id, self.parent_wbs_id, self.ev_user_pct, self.ev_etc_user_value,
               self.orig_cost, self.indep_remain_total_cost, self.ann_dscnt_rate_pct, self.dscnt_period_type,
               self.indep_remain_work_qty, self.anticip_start_date, self.anticip_end_date, self.ev_compute_type,
               self.ev_etc_compute_type, self.guid, self.tmpl_guid, self.plan_open_state]
        return tsv

    @classmethod
    def get_json(cls):
        root_nodes = list(filter(lambda x: WBS.find_by_id(x.parent_wbs_id) is None, cls.obj_list))
        print(root_nodes)
        json = dict()
        for node in root_nodes:
            json["node"] = node
            json["level"] = 0
            json["childs"] = []
            json["childs"].append(cls.get_childs(node, 0))
        print(json)
        return json

    @classmethod
    def get_childs(cls, node, level):
        nodes_lst = list(filter(lambda x: x.parent_wbs_id == node.wbs_id, cls.obj_list))
        nod = dict()
        for node in nodes_lst:
            nod["node"] = node
            nod["level"] = level + 1
            children = cls.get_childs(node, level + 1)
            nod["childs"] = []
            nod["childs"].append(children)
        return nod
    @classmethod
    def find_by_id(cls, ID):
        obj = list(filter(lambda x: x.wbs_id == ID, cls.obj_list))
        if obj:
            return obj[0]
        return None

    @classmethod
    def find_by_project_id(cls, project_id):
        return [ v for v in cls.obj_list if v.proj_id == project_id]

    @property
    def activities(self):
        return self.data.tasks.activities_by_wbs_id(self.wbs_id)

    def __repr__(self):
        return self.wbs_name



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


class SchedOption:
    obj_list = []

    def __init__(self, params):

        self.schedoptions_id = params.get('schedoptions_id').strip() if params.get('schedoptions_id') else None
        self.proj_id = params.get('proj_id').strip() if params.get('proj_id') else None
        self.sched_outer_depend_type = params.get('sched_outer_depend_type').strip() if params.get('sched_outer_depend_type') else None
        self.sched_open_critical_flag = params.get('sched_open_critical_flag').strip() if params.get('sched_open_critical_flag') else None
        self.sched_lag_early_start_flag = params.get('sched_lag_early_start_flag').strip() if params.get('sched_lag_early_start_flag') else None
        self.sched_retained_logic = params.get('sched_retained_logic').strip() if params.get('sched_retained_logic') else None
        self.sched_setplantoforecast = params.get('sched_setplantoforecast').strip() if params.get('sched_setplantoforecast') else None
        self.sched_float_type = params.get('sched_float_type').strip() if params.get('sched_float_type') else None
        self.sched_calendar_on_relationship_lag = params.get('sched_calendar_on_relationship_lag').strip() if params.get('sched_calendar_on_relationship_lag') else None
        self.sched_use_expect_end_flag = params.get('sched_use_expect_end_flag').strip() if params.get('sched_use_expect_end_flag') else None
        self.sched_progress_override = params.get('sched_progress_override').strip() if params.get('sched_progress_override') else None
        self.level_float_thrs_cnt = params.get('level_float_thrs_cnt').strip() if params.get('level_float_thrs_cnt') else None
        self.level_outer_assign_flag = params.get('level_outer_assign_flag').strip() if params.get('level_outer_assign_flag') else None
        self.level_outer_assign_priority = params.get('level_outer_assign_priority').strip() if params.get('level_outer_assign_priority') else None
        self.level_over_alloc_pct = params.get('level_over_alloc_pct').strip() if params.get('level_over_alloc_pct') else None
        self.level_within_float_flag = params.get('level_within_float_flag').strip() if params.get('level_within_float_flag') else None
        self.level_keep_sched_date_flag = params.get('level_keep_sched_date_flag').strip() if params.get('level_keep_sched_date_flag') else None
        self.level_all_rsrc_flag = params.get('level_all_rsrc_flag').strip() if params.get('level_all_rsrc_flag') else None
        self.sched_use_project_end_date_for_float = params.get('sched_use_project_end_date_for_float').strip() if params.get('sched_use_project_end_date_for_float') else None
        self.enable_multiple_longest_path_calc = params.get('enable_multiple_longest_path_calc').strip() if params.get('enable_multiple_longest_path_calc') else None
        self.limit_multiple_longest_path_calc = params.get('limit_multiple_longest_path_calc').strip() if params.get('limit_multiple_longest_path_calc') else None
        self.max_multiple_longest_path = params.get('max_multiple_longest_path').strip() if params.get('max_multiple_longest_path') else None
        self.use_total_float_multiple_longest_paths = params.get('use_total_float_multiple_longest_paths').strip() if params.get('use_total_float_multiple_longest_paths') else None
        self.key_activity_for_multiple_longest_paths = params.get('key_activity_for_multiple_longest_paths').strip() if params.get('use_total_float_multiple_longest_paths') else None
        self.LevelPriorityList = params.get('LevelPriorityList').strip() if params.get('LevelPriorityList') else None
        SchedOption.obj_list.append(self)

    def get_id(self):
        return self.schedoptions_id

    def get_tsv(self):
        tsv = ['%R', self.schedoptions_id, self.proj_id, self.sched_outer_depend_type, self.sched_open_critical_flag,
               self.sched_lag_early_start_flag, self.sched_retained_logic, self.sched_setplantoforecast,
               self.sched_float_type, self.sched_calendar_on_relationship_lag, self.sched_use_expect_end_flag,
               self.sched_progress_override, self.level_float_thrs_cnt, self.level_outer_assign_flag,
               self.level_outer_assign_priority, self.level_over_alloc_pct, self.level_within_float_flag,
               self.level_keep_sched_date_flag, self.level_all_rsrc_flag, self.sched_use_project_end_date_for_float,
               self.enable_multiple_longest_path_calc, self.limit_multiple_longest_path_calc,
               self.max_multiple_longest_path, self.use_total_float_multiple_longest_paths,
               self.key_activity_for_multiple_longest_paths, self.LevelPriorityList
               ]
        return tsv

    def __repr__(self):
        return self.proj_id


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


class ActivityStatus:

    Complete = "TK_Complete"
    InProgress = "TK_Active"
    NotStarted = "TK_NotStart"


class ActivityType:

    Task = "TT_Task"
    StartMilestone = "TT_Mile"
    FinishMilestone = "TT_FinMile"
    LevelOfEffort = "TT_LOE"
    WBSSummary = "TT_WBS"


class RelationshipType:

    FF = "PR_FF"
    FS = "PR_FS"
    SS = "PR_SS"
    SF = "PR_SF"


class DurationTypes:

    FixedQuantity = "DT_FixedQty"
    FixedDurationAndUnits = "DT_FixedDrtn"
    FixedRate = "DT_FixedRate"
    FixedDuration = "DT_FixedDUR2"



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.rsrc import Resource


class Resources:

    def __init__(self):
        self.index = 0
        self._rsrcs = []

    def add(self, params):
        rsrc = Resource(params)
        self._rsrcs.append(rsrc)

    def get_resource_by_id(self, id) -> Resource:
        rsrc = list(filter(lambda x: x.rsrc_id == id, self._rsrcs))
        if len(rsrc) > 0:
            rsrc = rsrc[0]
        else:
            rsrc = None
        return rsrc

    def get_parent(self, id) -> Resource:
        rsrc = list(filter(lambda x: x.rsrc_id == id, self._rsrcs))
        if len(rsrc) > 0:
            rsrc = rsrc[0]
            parent = Resources.get_resource_by_id(rsrc.parent_rsrc_id)
        else:
            rsrc = None
        return rsrc


    def __iter__(self):
        return self

    def __next__(self) -> Resource:
        if self.index >= len(self._rsrcs):
            raise StopIteration
        idx = self.index
        self.index += 1
        return self._rsrcs[idx]

    def _get_list(self):
        resor = []
        for res in self._rsrcs:
            resor.append((res.rsrc_id, res.parent_rsrc_id))
        return resor
    
    def get_tsv(self):
        tsv = []
        if len(self._rsrcs) > 0:
            tsv.append(['%T', 'RSRC'])
            tsv.append(['%F', 'rsrc_id', 'parent_rsrc_id', 'clndr_id',
               'role_id', 'shift_id', 'user_id', 'pobs_id',
               'guid', 'rsrc_seq_num', 'email_addr', 'employee_code',
               'office_phone', 'other_phone', 'rsrc_name', 'rsrc_short_name',
               'rsrc_title_name', 'def_qty_per_hr', 'cost_qty_type', 'ot_factor',
               'active_flag', 'auto_compute_act_flag', 'def_cost_qty_link_flag',
               'ot_flag', 'curr_id', 'unit_id', 'rsrc_type', 'location_id', 'rsrc_notes',
               'load_tasks_flag', 'level_flag', 'last_checksum'])
            for rsr in self._rsrcs:
                tsv.append(rsr.get_tsv())
        return tsv
    
    def build_tree(self):
        # pass 1: create nodes dictionary
        a = self._get_list()
        nodes = {}
        for i in a:
            id, parent_id = i
            nodes[id] = {id: self.get_resource_by_id(id)}
        # a = a[1:]
        # pass 2: create trees and parent-child relations
        forest = []
        for i in a:
            id, parent_id = i
            node = nodes[id]
            # either make the node a new tree or link it to its parent
            if parent_id == None or nodes.get(parent_id) == None:
                # start a new tree in the forest
                forest.append(node)
            else:
                # add new_node as child to parent
                parent = nodes.get(parent_id)

                if not 'children' in parent:
                    # ensure parent has a 'children' field
                    parent['children'] = []
                children = parent['children']
                children.append(node)
        return forest



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


class RCatType:
    obj_list = []

    def __init__(self, params):
        self.rsrc_catg_type_id = int(params.get('rsrc_catg_type_id').strip()) if params.get('rsrc_catg_type_id') else None
        self.seq_num = params.get('seq_num').strip() if params.get('seq_num') else None
        self.rsrc_catg_short_len = params.get('rsrc_catg_short_len').strip() if params.get('rsrc_catg_short_len') else None
        self.rsrc_catg_type = params.get('rsrc_catg_type').strip() if params.get('rsrc_catg_type') else None
        RCatType.obj_list.append(self)

    def get_tsv(self):
        tsv = ['%R', self.rsrc_catg_type_id,self.seq_num, self.rsrc_catg_short_len,
               self.rsrc_catg_type]
        return tsv

    def get_id(self):
        return self.rsrc_catg_type_id

    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.rsrc_catg_type_id == id, cls.obj_list))
        if obj:
            return obj[0]
        return obj

    def __repr__(self):
        return self.rsrc_catg_type


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.calendar import Calendar
from xerparser.model.classes.activitycode import ActivityCode
from xerparser.model.classes.acttype import ActType
from xerparser.model.classes.obs import OBS
from xerparser.model.classes.project import Project
from xerparser.model.classes.rcattype import RCatType
from xerparser.model.classes.rcatval import RCatVal
from xerparser.model.classes.rsrc import Resource
from xerparser.model.classes.rsrcrate import ResourceRate
from xerparser.model.classes.rsrcrcat import ResourceCat
from xerparser.model.classes.schedoption import SchedOption
from xerparser.model.classes.task import Task
from xerparser.model.classes.taskactv import TaskActv
from xerparser.model.classes.taskpred import TaskPred
from xerparser.model.classes.udftype import UDFType
from xerparser.model.classes.udfvalue import UDFValue
from xerparser.model.classes.wbs import WBS
from xerparser.model.classes.currency import Currency
from xerparser.model.classes.taskrsrc import TaskRsrc
from xerparser.model.classes.role import Role
from xerparser.model.classes.account import Account
from xerparser.model.classes.rolerate import RoleRate
from xerparser.model.classes.pcattype import PCatType
from xerparser.model.classes.pcatval import PCatVal
from xerparser.model.classes.projcat import ProjCat
from xerparser.model.classes.taskproc import TaskProc
from xerparser.model.classes.fintmpl import FinTmpl
from xerparser.model.classes.nonwork import NonWork
from xerparser.model.tasks import Tasks
from xerparser.model.predecessors import Predecessors
from xerparser.model.projects import Projects
from xerparser.model.wbss import WBSs
from xerparser.model.resources import Resources
from xerparser.model.accounts import Accounts
from xerparser.model.activitycodes import ActivityCodes
from xerparser.model.acttypes import ActTypes
from xerparser.model.calendars import Calendars
from xerparser.model.currencies import Currencies
from xerparser.model.obss import OBSs
from xerparser.model.rcattypes import RCatTypes
from xerparser.model.rcatvals import RCatVals
from xerparser.model.rolerates import RoleRates
from xerparser.model.roles import Roles
from xerparser.model.rsrccurves import ResourceCurves
from xerparser.model.rsrcrates import ResourceRates
from xerparser.model.rsrccats import ResourceCategories
from xerparser.model.schedoptions import SchedOptions
from xerparser.model.activitiyresources import ActivityResources
from xerparser.model.udfvalues import UDFValues
from xerparser.model.udftypes import UDFTypes
from xerparser.model.taskactvs import TaskActvs
from xerparser.model.pacttypes import PCatTypes
from xerparser.model.pcatvals import PCatVals
from xerparser.model.projcats import ProjCats
from xerparser.model.taskprocs import TaskProcs
from xerparser.model.fintmpls import FinTmpls
from xerparser.model.nonworks import NonWorks



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.taskpred import TaskPred
# from xerparser.model.classes.task import Task
from typing import List


class Predecessors:

    def __init__(self):
        self.index=0
        self.task_pred = []

    def find_by_id(self, code_id) -> TaskPred:
        obj = list(filter(lambda x: x.task_pred_id == code_id, self.task_pred))
        if len(obj) > 0:
            obj[0]
        else:
            obj = None
        return obj
    
    def get_tsv(self):
        tsv = []
        if len(self.task_pred) > 0:
            tsv.append(['%T', 'TASKPRED'])
            tsv.append(['%F', 'task_pred_id', 'task_id', 'pred_task_id', 'proj_id', 'pred_proj_id',
               'pred_type', 'lag_hr_cnt', 'comments', 'float_path', 'aref', 'arls'])
            for pred in self.task_pred:
                tsv.append(pred.get_tsv())
        return tsv
        
    def add(self, params):
        pred = TaskPred(params)
        self.task_pred.append(pred)

    @property
    def relations(self) -> List[TaskPred]:
        return self.task_pred

    @property
    def leads(self):
        return list(filter(lambda x: x.lag_hr_cnt < 0 if x.lag_hr_cnt else None, self.task_pred))

    @property
    def finish_to_start(self) -> List[TaskPred]:
        return list(filter(lambda x: x.pred_type == 'PR_FS', self.task_pred))
    
    def get_successors(self, act_id):
        succ = list(filter(lambda x: x.pred_task_id == act_id, self.task_pred))
        return succ

    def get_predecessors(self, act_id):
        succ = list(filter(lambda x: x.task_id == act_id, self.task_pred))
        return succ

    def count(self):
        return len(self.task_pred)

    def __len__(self):
        return len(self.task_pred)

    def __iter__(self):
        return self

    def __next__(self) -> TaskPred:
        if self.index >= len(self.task_pred):
            raise StopIteration
        idx = self.index
        self.index += 1
        return self.task_pred[idx]




# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


class PCatVal:

    def __init__(self, params):
        # %F	proj_catg_id	proj_catg_type_id	seq_num	proj_catg_short_name	parent_proj_catg_id	proj_catg_name
        self.proj_catg_id = params.get('proj_catg_id').strip() if params.get('proj_catg_id') else None
        self.proj_catg_type_id = params.get('proj_catg_type_id').strip() if params.get('proj_catg_type_id') else None
        self.seq_num = params.get('seq_num').strip() if params.get('seq_num') else None
        self.proj_catg_short_name = params.get('proj_catg_short_name').strip() if params.get(
            'proj_catg_short_name') else None
        self.parent_proj_catg_id = params.get('parent_proj_catg_id').strip() if params.get(
            'parent_proj_catg_id') else None
        self.proj_catg_name = params.get('proj_catg_name').strip() if params.get('proj_catg_name') else None


    def get_id(self):
        return self.proj_catg_id

    def get_tsv(self):
        tsv = ['%R', self.proj_catg_id, self.proj_catg_type_id, self.seq_num, self.proj_catg_short_name,
               self.parent_proj_catg_id, self.proj_catg_name]
        return tsv

    def __repr__(self):
        return self.proj_catg_name



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.
from datetime import datetime
from logging import critical


class DCMA14():
    """

    {Analysis :
        {
        Predecessors: {
            cnt : 45
            tasks: []
            }
        },
        successors: {
            cnt: 30,
            tasks: []
        }
    """
    def __init__(self, programme, duration_limit=1, lag_limit=0, tf_limit=0):
        self.count = 0
        self.programme = programme
        self.dur_limit = duration_limit
        self.lag_limit = lag_limit
        self.tf_limit = tf_limit
        self.results = {}
        self.results['analysis'] = {}


    def analysis(self):
        self.activity_count = len(self.programme.activities)
        self.relation_count = len(self.programme.relations)
        self.results['analysis']['summary'] = {'activity_cnt': self.activity_count, 'relationship_cnt': self.relation_count}
        # 1.1 successors
        self.no_successors = self.chk_successors()
        self.no_successors_cnt = len(self.no_successors)
        self.results['analysis']['successors'] = {'cnt': self.no_successors_cnt,
                                                  'activities': [self.get_activity(x.task_id) for x in self.no_successors],
                                                  'pct': self.no_successors_cnt / float(self.activity_count)}
        #1.2 predecessors
        self.no_predecessors = self.chk_predessors()
        self.no_predecessors_cnt = len(self.no_predecessors)
        self.results['analysis']['predecessors'] = {'cnt': self.no_predecessors_cnt,
                                                    'activities': [self.get_activity(x.task_id) for x in self.no_predecessors],
                                                    'pct': self.no_predecessors_cnt / float(self.activity_count)}
        #2 lags
        self.lags = list(filter(lambda x: x.lag_hr_cnt > self.lag_limit if x.lag_hr_cnt else None, self.programme.relations))
        self.results['analysis']['lags'] = {'cnt': len(self.lags),
                                            'relations': [ { 
                                                            "successor":self.get_activity(x.task_id),
                                                            "predecessor":self.get_activity(x.pred_task_id),
                                                            "type": x.pred_type,
                                                            "lag": int(x.lag_hr_cnt / 8.0)
                                                            } for x in self.lags],
                                            'pct': len(self.lags) / float(self.relation_count)}
        #3 leads
        self.leads = self.programme.relations.leads
        self.results['analysis']['leads'] = {'cnt': len(self.leads),
                                             'relations': [{ 
                                                            "successor":self.get_activity(x.task_id),
                                                            "predecessor":self.get_activity(x.pred_task_id),
                                                            "type": x.pred_type,
                                                            "lag": int(x.lag_hr_cnt / 8.0)
                                                            } for x in self.leads],
                                             'pct': len(self.leads) / float(self.relation_count)}
        #4 relationships
        self.fsRel = self.programme.relations.finish_to_start
        self.results['analysis']['relations'] = {'fs_cnt': len(self.fsRel), 'relationship': [
                                                { 
                                                    "successor":self.get_activity(x.task_id),
                                                    "predecessor":self.get_activity(x.pred_task_id),
                                                    "type": x.pred_type,
                                                    "lag": int(x.lag_hr_cnt / 8.0)
                                                    } for x in self.fsRel]}
        #5 constraints
        lst = ['CS_MANDFIN', 'CS_MANDFIN']
        self.constraints = list(filter(lambda x: x.get('ConstraintType') in lst,
                                        self.programme.activities))
        self.results['analysis']['constraints'] = {'cstr_cnt': len(self.constraints), 
                                                    'cstrs': [self.get_activity(x.task_id) for x in self.constraints]}
        #6 large total float
        self.totalfloat = list(filter(lambda x: x.total_float_hr_cnt /8.0 > self.tf_limit if x.total_float_hr_cnt else 0,  self.programme.activities.activities))
        self.results['analysis']['totalfloat'] = {'cnt': len(self.totalfloat),
                                                'activities': [self.get_activity(x.task_id) for x in self.totalfloat],
                                                'pct': len(self.totalfloat) / float(self.activity_count)}
        #7 negative total float
        self.negativefloat = list(filter(lambda x: x.total_float_hr_cnt /8.0 < 0 if x.total_float_hr_cnt else 0,  self.programme.activities.activities))
        self.results['analysis']['negativefloat'] = {'cnt': len(self.negativefloat),
                                                'activities': [self.get_activity(x.task_id) for x in self.negativefloat],
                                                'pct': len(self.negativefloat) / float(self.activity_count)}
        #8 durations
        self.duration = list(filter(lambda x: x.duration > self.dur_limit,  self.programme.activities.activities))
        self.results['analysis']['duration'] = {'cnt': len(self.duration),
                                                'activities': [self.get_activity(x.task_id) for x in self.duration],
                                                'pct': len(self.duration) / float(self.activity_count)}
        #9 Check for Invalid Dates
        # no actual dates beyong data date
        data_date = {}
        for x in self.programme.projects:
            data_date[str(x.proj_id)] = datetime.strptime(x.last_recalc_date, "%Y-%m-%d %H:%M")
        print(data_date)
        self.invalidactualstart = list(filter(lambda x: None if x.act_start_date is None else x.act_start_date > data_date[str(x.proj_id)], self.programme.activities.activities))
        self.invalidactualfinish = list(filter(lambda x: None if x.act_end_date is None else x.act_end_date > data_date[str(x.proj_id)], self.programme.activities.activities))
        self.invalidearlystart = list(filter(lambda x: None if x.early_start_date is None else x.early_start_date < data_date[str(x.proj_id)], self.programme.activities.activities))
        self.invalidearlyfinish = list(filter(lambda x: None if x.early_end_date is None else x.early_end_date < data_date[str(x.proj_id)], self.programme.activities.activities))
        cnt = len(self.invalidactualfinish) + len(self.invalidactualstart) + len(self.invalidearlystart) + len(self.invalidearlyfinish)
        pct = cnt / float(self.activity_count)
        self.invaliddates = {
            "actual_start": [self.get_activity(x.task_id) for x in self.invalidactualstart],
            "actual_finish": [self.get_activity(x.task_id) for x in self.invalidactualfinish],
            "early_start": [self.get_activity(x.task_id) for x in self.invalidearlystart],
            "early_finish": [self.get_activity(x.task_id) for x in self.invalidearlyfinish],
            "cnt": cnt,
            'pct': pct
            }
        self.results['analysis']['invaliddates'] = self.invaliddates

        #10 Check resource assignments
        no_resources = []
        tasks_id = [x.task_id for x in self.programme.activities.activities]
        for t_id in tasks_id:
            assignments = self.programme.activityresources.find_by_activity_id(t_id)
            if len(assignments) == 0:
                no_resources.append(t_id)
        self.results['analysis']['resources'] = {
            'activities': [self.get_activity(x) for x in no_resources],
            "cnt": len(no_resources),
            'pct': len(no_resources) / float(self.activity_count)
        }
        print(no_resources)

        #11 slippage from target
        # end dates are later than target end dates
        self.actualendslippage = list(filter(lambda x: None if x.act_end_date is None else x.act_end_date > x.target_end_date, self.programme.activities.activities))
        self.earlyendslippage = list(filter(lambda x: None if x.early_end_date is None else x.early_end_date > x.target_end_date, self.programme.activities.activities))
        slipped = self.actualendslippage + self.earlyendslippage
        print("SLIPPED", slipped)
        self.results['analysis']['slippage'] = {
            'activities': [{
                "id":x.task_code,
                "name": x.task_name,
                "early_finish": str(x.early_end_date),
                "planned_finish": str(x.target_end_date)
            } for x in slipped],
            'cnt': len(slipped),
            'pct': len(slipped) / float(self.activity_count)
        }

        #12 Critical Path Test
         
        
        #13 Critical Path Length Index
        # calculated as cirical path length + total float / critical path length
        # critical = list(filter(lambda x: x.total_float_hr_cnt <= 10.0 if x.total_float_hr_cnt else None, self.programme.activities.activities))
        critical = []
        for act in self.programme.activities.activities:
            if act.total_float_hr_cnt is not None:
                print("TF FOUND", act.task_code, act.total_float_hr_cnt)
                if act.total_float_hr_cnt <= 0:
                    
                    critical.append(act)
            else:
                print("TF Not found")


        print("critical", [(task.task_code, task.early_start_date, task.total_float_hr_cnt) for task in critical])

        self.results['analysis']['critical'] = {
            'activities': [self.get_activity(x.task_id) for x in critical],
            'cnt': len(critical),
            'pct': len(critical) / self.activity_count
        }

        

        #14 BLEI


        return self.results

    def chk_successors(self):
        return self.programme.activities.has_no_successor

    def chk_predessors(self):
        return self.programme.activities.has_no_predecessor

    def get_activity(self, id):
        activity = self.programme.activities.find_by_id(id)
        # print(activity)
        if type(activity) == list:
            return None
        return {
                "id": activity.task_code, 
                "name": activity.task_name,
                "duration": activity.duration,
                "tf":activity.total_float_hr_cnt / 8.0 if activity.total_float_hr_cnt else 0
                }



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


import csv

def writeXER(r, filename):
    header = ['ERMHDR', '8.0', '2021-11-02', 'Project',	'admin', 'Primavera', 'Admin', 'dbxDatabaseNoName', 'Project Management', 'U.K.']
    with open(filename, 'w', newline='', encoding='utf-8') as output:
        tsv_writer = csv.writer(output, delimiter='\t')
        tsv_writer.writerow(header)
        tsv_writer.writerows(r.currencies.get_tsv())
        tsv_writer.writerows(r.fintmpls.get_tsv())
        tsv_writer.writerows(r.nonworks.get_tsv())
        tsv_writer.writerows(r.obss.get_tsv())
        tsv_writer.writerows(r.pcattypes.get_tsv())
        tsv_writer.writerows(r.resourcecurves.get_tsv())
        tsv_writer.writerows(r.udftypes.get_tsv())
        tsv_writer.writerows(r.accounts.get_tsv())
        tsv_writer.writerows(r.pcatvals.get_tsv())
        tsv_writer.writerows(r.projects.get_tsv())
        tsv_writer.writerows(r.calendars.get_tsv())
        tsv_writer.writerows(r.projpcats.get_tsv())
        tsv_writer.writerows(r.scheduleoptions.get_tsv())
        tsv_writer.writerows(r.wbss.get_tsv())
        tsv_writer.writerows(r.resources.get_tsv())
        tsv_writer.writerows(r.acttypes.get_tsv())
        tsv_writer.writerows(r.resourcerates.get_tsv())
        tsv_writer.writerows(r.activities.get_tsv())
        tsv_writer.writerows(r.actvcodes.get_tsv())
        # PROJCOST
        tsv_writer.writerows(r.relations.get_tsv())
        tsv_writer.writerows(r.taskprocs.get_tsv())
        tsv_writer.writerows(r.activityresources.get_tsv())
        tsv_writer.writerows(r.activitycodes.get_tsv())
        tsv_writer.writerows(r.udfvalues.get_tsv())
        tsv_writer.writerow(['%E'])




# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.acttypes import ActTypes
import locale

class ActivityCode:
    obj_list = []

    def __init__(self, params):
        # Unique ID generated by the system.
        self.actv_code_id = int(params.get('actv_code_id').strip()) if params.get('actv_code_id') else None
        # The parent Activity Code value in the user code value hierarchy.
        self.parent_actv_code_id = int(params.get('parent_actv_code_id')) if params.get('parent_actv_code_id') else None
        # The Activity Code type id acts as foreign key to activity code types table
        self.actv_code_type_id = int(params.get('actv_code_type_id').strip()) if params.get('actv_code_type_id') else None
        # The description of an Activity Code value.
        self.actv_code_name = params.get('actv_code_name').strip() if params.get('actv_code_name') else None
        # The value of the activity code.
        self.short_name = params.get('short_name').strip() if params.get('short_name') else None
        # Sequence number for sorting.
        self.seq_num = int(params.get('seq_num').strip()) if params.get('seq_num') else None
        self.color = params.get('color').strip() if params.get('color') else None
        self.total_assignments = int(locale.atof(params.get('total_assignments').strip())) if params.get('total_assignments') else None
        ActivityCode.obj_list.append(self)

    def get_id(self):
        return self.actv_code_id

    def get_tsv(self):
        tsv = ['%R', self.actv_code_id, self.parent_actv_code_id, self.actv_code_type_id,
               self.actv_code_name, self.short_name, self.seq_num, self.color, self.total_assignments]
        return tsv

    # @staticmethod
    # def find_by_id(id):
    #     obj = list(filter(lambda x: x.actv_code_id == id, ActivityCode.obj_list))
    #     if obj:
    #         return obj[0]
    #     return obj

    # @classmethod
    # def find_by_code(cls, code):
    #     """ This Function searches for activity code using ID code
    #
    #     Args:
    #         code: ID code as defined in Primavera and not the database
    #         obj_list: list of activity codes that need to be searched
    #
    #     Returns: an ActivityCode object that matches the supplied code
    #
    #     """
    #     actv_code = list(filter(lambda x: x.short_name == code, ActivityCode.obj_list))[0]
    #     return actv_code
    #
    # @classmethod
    # def get_parent(cls, id):
    #     obj = None
    #     c_actv_code = list(filter(lambda x: x.actv_code_id == id, ActivityCode.obj_list))[0]
    #     if c_actv_code:
    #         obj = cls.find_by_id(c_actv_code.actv_code_id, ActivityCode.obj_list)
    #     return obj
    #
    # @classmethod
    # def get_children(cls, id, obj_list):
    #     childs = list(filter(lambda x: x.parent_actv_code_id == id, obj_list))
    #     return childs

    @property
    def type(self):
        return ActTypes.find_by_id(self.actv_code_type_id)

    def __repr__(self):
        return str(self.actv_code_id) + ' - ' + self.short_name if self.short_name else '' + ' - ' + self.actv_code_name if self.actv_code_name else ''


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


class UDFType:

    udf_type_id = None
    table_name = None
    udf_type_name = None
    udf_type_label = None
    logical_data_type = None
    super_flag = None
    indicator_expression = None
    summary_indicator_expression = None

    def __init__(self, params):
        # %F	udf_type_id	table_name	udf_type_name	udf_type_label	logical_data_type
        # super_flag	indicator_expression	summary_indicator_expression	export_flag
        self.udf_type_id = params.get('udf_type_id').strip() if params.get('udf_type_id') else None
        self.table_name = params.get('table_name').strip() if params.get('table_name') else None
        self.udf_type_name = params.get('udf_type_name').strip() if params.get('udf_type_name') else None
        self.udf_type_label = params.get('udf_type_label').strip() if params.get('udf_type_label') else None
        self.logical_data_type = params.get('logical_data_type').strip() if params.get('logical_data_type') else None
        self.super_flag = params.get('super_flag').strip() if params.get('super_flag') else None
        self.indicator_expression = params.get('indicator_expression').strip() if params.get(
            'indicator_expression') else None
        self.summary_indicator_expression = params.get('summary_indicator_expression').strip() if params.get(
            'summary_indicator_expression') else None
        self.export_flag = params.get('export_flag').strip() if params.get('export_flag') else None

    def get_id(self):
        return self.udf_type_id

    def get_tsv(self):
        tsv = ['%R', self.udf_type_id, self.table_name, self.udf_type_name, self.udf_type_label, self.logical_data_type,
               self.super_flag, self.indicator_expression, self.summary_indicator_expression, self.export_flag]
        return tsv

    def __repr__(self):
        return self.udf_type_name



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.schedoption import SchedOption


class SchedOptions:

    def __init__(self):
        self.index = 0
        self._schoptions = []

    def add(self, params):
        self._schoptions.append(SchedOption(params))

    def find_by_id(self, id) -> SchedOption:
        obj = list(filter(lambda x: x.actv_code_type_id == id, self._schoptions))
        if len(obj) > 0:
            return obj[0]
        return obj
    
    def get_tsv(self):
        tsv = []
        if len(self._schoptions) > 0:
            tsv.append(['%T', 'SCHEDOPTIONS'])
            tsv.append(['%F', 'schedoptions_id', 'proj_id', 'sched_outer_depend_type', 'sched_open_critical_flag',
               'sched_lag_early_start_flag', 'sched_retained_logic', 'sched_setplantoforecast',
               'sched_float_type', 'sched_calendar_on_relationship_lag', 'sched_use_expect_end_flag',
               'sched_progress_override', 'level_float_thrs_cnt', 'level_outer_assign_flag',
               'level_outer_assign_priority', 'level_over_alloc_pct', 'level_within_float_flag',
               'level_keep_sched_date_flag', 'level_all_rsrc_flag', 'sched_use_project_end_date_for_float',
               'enable_multiple_longest_path_calc', 'limit_multiple_longest_path_calc',
               'max_multiple_longest_path', 'use_total_float_multiple_longest_paths',
               'key_activity_for_multiple_longest_paths', 'LevelPriorityList'])
            for sco in self._schoptions:
                tsv.append(sco.get_tsv())
        return tsv

    @property
    def count(self):
        return len(self._schoptions)

    def __len__(self):
        return len(self._schoptions)

    def __iter__(self):
        return self

    def __next__(self) -> SchedOption:
        if self.index >= len(self._schoptions):
            raise StopIteration
        idx = self.index
        self.index += 1
        return self._schoptions[idx]



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.
import locale

class ResourceCurve:
    obj_list = []

    def __init__(self, params):        
        self.curv_id = int(params.get('curv_id')) if params.get('curv_id') else None
        self.curv_name = params.get('curv_name').strip() if params.get('curv_id') else None
        self.default_flag = params.get('default_flag') if params.get('default_flag') else None
        self.pct_usage_0 = locale.atof(params.get('pct_usage_0')) if params.get('pct_usage_0') else None
        self.pct_usage_1 = locale.atof(params.get('pct_usage_1')) if params.get('pct_usage_1') else None
        self.pct_usage_2 = locale.atof(params.get('pct_usage_2')) if params.get('pct_usage_2') else None
        self.pct_usage_3 = locale.atof(params.get('pct_usage_3')) if params.get('pct_usage_3') else None
        self.pct_usage_4 = locale.atof(params.get('pct_usage_4')) if params.get('pct_usage_4') else None
        self.pct_usage_5 = locale.atof(params.get('pct_usage_5')) if params.get('pct_usage_5') else None
        self.pct_usage_6 = locale.atof(params.get('pct_usage_6')) if params.get('pct_usage_6') else None
        self.pct_usage_7 = locale.atof(params.get('pct_usage_7')) if params.get('pct_usage_7') else None
        self.pct_usage_8 = locale.atof(params.get('pct_usage_8')) if params.get('pct_usage_8') else None
        self.pct_usage_9 = locale.atof(params.get('pct_usage_9')) if params.get('pct_usage_9') else None
        self.pct_usage_10 = locale.atof(params.get('pct_usage_10')) if params.get('pct_usage_10') else None
        self.pct_usage_11 = locale.atof(params.get('pct_usage_11')) if params.get('pct_usage_11') else None
        self.pct_usage_12 = locale.atof(params.get('pct_usage_12')) if params.get('pct_usage_12') else None
        self.pct_usage_13 = locale.atof(params.get('pct_usage_13')) if params.get('pct_usage_13') else None
        self.pct_usage_14 = locale.atof(params.get('pct_usage_14')) if params.get('pct_usage_14') else None
        self.pct_usage_15 = locale.atof(params.get('pct_usage_15')) if params.get('pct_usage_15') else None
        self.pct_usage_16 = locale.atof(params.get('pct_usage_16')) if params.get('pct_usage_16') else None
        self.pct_usage_17 = locale.atof(params.get('pct_usage_17')) if params.get('pct_usage_17') else None
        self.pct_usage_18 = locale.atof(params.get('pct_usage_18')) if params.get('pct_usage_18') else None
        self.pct_usage_19 = locale.atof(params.get('pct_usage_19')) if params.get('pct_usage_19') else None
        self.pct_usage_20 = locale.atof(params.get('pct_usage_20')) if params.get('pct_usage_20') else None

        ResourceCurve.obj_list.append(self)

    def get_tsv(self):
        tsv = ['%R', self.curv_id, self.curv_name, self.default_flag, self.pct_usage_0,
               self.pct_usage_1, self.pct_usage_2, self.pct_usage_3, self.pct_usage_4,
               self.pct_usage_5, self.pct_usage_6, self.pct_usage_7, self.pct_usage_8,
               self.pct_usage_9, self.pct_usage_10, self.pct_usage_11, self.pct_usage_12,
               self.pct_usage_13, self.pct_usage_14, self.pct_usage_15, self.pct_usage_16,
               self.pct_usage_17, self.pct_usage_18, self.pct_usage_19, self.pct_usage_20]
        return tsv

    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.curv_id == id, cls.obj_list))[0]
        return obj

    def __repr__(self):
        return self.curv_name



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.wbs import WBS
from xerparser.model.tasks import Tasks
from xerparser.model.wbss import WBSs

class Project:

    def __init__(self, params, data):

        self.proj_id = int(params.get('proj_id').strip()) if params.get('proj_id') else None
        self.fy_start_month_num = params.get('fy_start_month_num').strip() if params.get('fy_start_month_num') else None
        self.rsrc_self_add_flag = params.get('rsrc_self_add_flag').strip() if params.get('rsrc_self_add_flag') else None
        self.allow_complete_flag = params.get('allow_complete_flag').strip() if params.get('allow_complete_flag') else None
        self.rsrc_multi_assign_flag = params.get('rsrc_multi_assign_flag').strip() if params.get('rsrc_multi_assign_flag') else None
        self.checkout_flag = params.get('checkout_flag').strip() if params.get('checkout_flag') else None
        self.project_flag = params.get('project_flag').strip() if params.get('project_flag') else None
        self.step_complete_flag = params.get('step_complete_flag').strip() if params.get('step_complete_flag') else None
        self.cost_qty_recalc_flag = params.get('cost_qty_recalc_flag').strip() if params.get('cost_qty_recalc_flag') else None
        self.batch_sum_flag = params.get('batch_sum_flag').strip() if params.get('batch_sum_flag') else None
        self.name_sep_char = params.get('name_sep_char').strip() if params.get('name_sep_char') else None
        self.def_complete_pct_type = params.get('def_complete_pct_type').strip() if params.get('def_complete_pct_type') else None
        self.proj_short_name = params.get('proj_short_name').strip() if params.get('proj_short_name') else None
        self.acct_id = params.get('acct_id').strip() if params.get('acct_id') else None
        self.orig_proj_id = params.get('orig_proj_id').strip() if params.get('orig_proj_id') else None
        self.source_proj_id = params.get('source_proj_id').strip() if params.get('source_proj_id') else None
        self.base_type_id = params.get('base_type_id').strip() if params.get('base_type_id') else None
        self.clndr_id = params.get('clndr_id').strip() if params.get('clndr_id') else None
        self.sum_base_proj_id = params.get('sum_base_proj_id').strip() if params.get('sum_base_proj_id') else None
        self.task_code_base = params.get('task_code_base').strip() if params.get('task_code_base') else None
        self.task_code_step = params.get('task_code_step').strip() if params.get('task_code_step') else None
        self.priority_num = params.get('priority_num').strip() if params.get('priority_num') else None
        self.wbs_max_sum_level = params.get('wbs_max_sum_level').strip() if params.get('wbs_max_sum_level') else None
        self.strgy_priority_num = params.get('strgy_priority_num').strip() if params.get('strgy_priority_num') else None
        self.last_checksum = params.get('last_checksum').strip() if params.get('last_checksum') else None
        self.critical_drtn_hr_cnt = params.get('critical_drtn_hr_cnt').strip() if params.get('critical_drtn_hr_cnt') else None
        self.def_cost_per_qty = params.get('def_cost_per_qty').strip() if params.get('def_cost_per_qty') else None
        self.last_recalc_date = params.get('last_recalc_date').strip() if params.get('last_recalc_date') else None
        self.plan_start_date = params.get('plan_start_date').strip() if params.get('plan_start_date') else None
        self.plan_end_date = params.get('plan_end_date').strip() if params.get('plan_end_date') else None
        self.scd_end_date = params.get('scd_end_date').strip() if params.get('scd_end_date') else None
        self.add_date = params.get('add_date').strip() if params.get('add_date') else None
        self.last_tasksum_date = params.get('last_tasksum_date').strip() if params.get('last_tasksum_date') else None
        self.fcst_start_date = params.get('fcst_start_date').strip() if params.get('fcst_start_date') else None
        self.def_duration_type = params.get('def_duration_type').strip() if params.get('def_duration_type') else None
        self.task_code_prefix = params.get('task_code_prefix').strip() if params.get('task_code_prefix') else None
        self.guid = params.get('guid').strip() if params.get('guid') else None
        self.def_qty_type = params.get('def_qty_type').strip() if params.get('def_qty_type') else None
        self.add_by_name = params.get('add_by_name').strip() if params.get('add_by_name') else None
        self.web_local_root_path = params.get('web_local_root_path').strip() if params.get('web_local_root_path') else None
        self.proj_url = params.get('proj_url').strip() if params.get('proj_url') else None
        self.def_rate_type = params.get('def_rate_type').strip() if params.get('def_rate_type') else None
        self.add_act_remain_flag = params.get('add_act_remain_flag').strip() if params.get('add_act_remain_flag') else None
        self.act_this_per_link_flag = params.get('act_this_per_link_flag').strip() if params.get('act_this_per_link_flag') else None
        self.def_task_type = params.get('def_task_type').strip() if params.get('def_task_type') else None
        self.act_pct_link_flag = params.get('act_pct_link_flag').strip() if params.get('act_pct_link_flag') else None
        self.critical_path_type = params.get('critical_path_type').strip() if params.get('critical_path_type') else None
        self.task_code_prefix_flag = params.get('task_code_prefix_flag').strip() if params.get('task_code_prefix_flag') else None
        self.def_rollup_dates_flag = params.get('def_rollup_dates_flag').strip() if params.get('def_rollup_dates_flag') else None
        self.use_project_baseline_flag = params.get('use_project_baseline_flag').strip() if params.get('use_project_baseline_flag') else None
        self.rem_target_link_flag = params.get('rem_target_link_flag').strip() if params.get('rem_target_link_flag') else None
        self.reset_planned_flag = params.get('reset_planned_flag').strip() if params.get('reset_planned_flag') else None
        self.allow_neg_act_flag = params.get('allow_neg_act_flag').strip() if params.get('allow_neg_act_flag') else None
        self.sum_assign_level = params.get('sum_assign_level').strip() if params.get('sum_assign_level') else None
        self.last_fin_dates_id = params.get('last_fin_dates_id').strip() if params.get('last_fin_dates_id') else None
        self.last_baseline_update_date = params.get('last_fin_dates_id').strip() if params.get('last_fin_dates_id') else None
        self.cr_external_key = params.get('cr_external_key').strip() if params.get('cr_external_key') else None
        self.apply_actuals_date = params.get('apply_actuals_date').strip() if params.get('apply_actuals_date') else None
        self.location_id = params.get('location_id') if params.get('location_id') else None
        self.loaded_scope_level = params.get('loaded_scope_level').strip() if params.get('loaded_scope_level') else None
        self.export_flag = params.get('export_flag').strip() if params.get('export_flag') else None
        self.new_fin_dates_id = params.get('new_fin_dates_id').strip() if params.get('new_fin_dates_id') else None
        self.baselines_to_export = params.get('baselines_to_export').strip() if params.get('baselines_to_export') else None
        self.baseline_names_to_export = params.get('baseline_names_to_export').strip() if params.get('baseline_names_to_export') else None
        self.next_data_date = params.get('next_data_date').strip() if params.get('next_data_date') else None
        self.close_period_flag = params.get('close_period_flag').strip() if params.get('close_period_flag') else None
        self.sum_refresh_date = params.get('sum_refresh_date').strip() if params.get('sum_refresh_date') else None
        self.trsrcsum_loaded = params.get('trsrcsum_loaded').strip() if params.get('trsrcsum_loaded') else None
        self.fintmpl_id = int(params.get('fintmpl_id').strip()) if params.get('fintmpl_id') else None
        self.sumtask_loaded = params.get('sumtask_loaded').strip() if params.get('sumtask_loaded') else None
        self.data = data

    @property
    def id(self):
        return self.proj_id

    def get_tsv(self):
        tsv = ['%R', self.proj_id, self.fy_start_month_num, self.rsrc_self_add_flag,
               self.allow_complete_flag, self.rsrc_multi_assign_flag, self.checkout_flag,
               self.project_flag, self.step_complete_flag, self.cost_qty_recalc_flag,
               self.batch_sum_flag, self.name_sep_char, self.def_complete_pct_type, self.proj_short_name,
               self.acct_id, self.orig_proj_id, self.source_proj_id, self.base_type_id, self.clndr_id,
               self.sum_base_proj_id, self.task_code_base, self.task_code_step, self.priority_num,
               self.wbs_max_sum_level, self.strgy_priority_num, self.last_checksum, self.critical_drtn_hr_cnt,
               self.def_cost_per_qty, self.last_recalc_date, self.plan_start_date, self.plan_end_date,
               self.scd_end_date, self.add_date, self.last_tasksum_date, self.fcst_start_date, self.def_duration_type,
               self.task_code_prefix, self.guid, self.def_qty_type, self.add_by_name, self.web_local_root_path,
               self.proj_url, self.def_rate_type, self.add_act_remain_flag, self.act_this_per_link_flag,
               self.def_task_type, self.act_pct_link_flag, self.critical_path_type, self.task_code_prefix_flag,
               self.def_rollup_dates_flag, self.use_project_baseline_flag, self.rem_target_link_flag,
               self.reset_planned_flag, self.allow_neg_act_flag, self.sum_assign_level, self.last_fin_dates_id,
               self.last_baseline_update_date, self.cr_external_key, self.apply_actuals_date, self.fintmpl_id,
               self.location_id, self.loaded_scope_level, self.export_flag, self.new_fin_dates_id,
               self.baselines_to_export, self.baseline_names_to_export, self.next_data_date, self.close_period_flag,
               self.sum_refresh_date, self.trsrcsum_loaded, self.sumtask_loaded]
        return tsv

    @property
    def activities(self):
        return self.data.tasks.get_by_project(self.proj_id)

    @property
    def wbss(self):
        # wbss = WBSs()
        return self.data.wbss.get_by_project(self.proj_id)

    def __repr__(self):
        return self.proj_short_name


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.pcatval import PCatVal


class PCatVals:

    def __init__(self):
        self.index = 0
        self._PCatVals = []

    def add(self, params):
        self._PCatVals.append(PCatVal(params))

    def get_tsv(self):
        tsv = []
        if len(self._PCatVals) > 0:
            tsv.append(['%T', 'PCATVAL'])
            tsv.append(['%F', 'proj_catg_id', 'proj_catg_type_id', 'seq_num', 'proj_catg_short_name',
               'parent_proj_catg_id', 'proj_catg_name'])
            for pcatval in self._PCatVals:
                tsv.append(pcatval.get_tsv())
        return tsv

    def find_by_id(self, id) -> PCatVal:
        obj = list(filter(lambda x: x.proj_catg_id == id, self._PCatVals))
        if len(obj) > 0:
            return obj[0]
        return obj

    @property
    def count(self):
        return len(self._PCatVals)

    def __len__(self):
        return len(self._PCatVals)

    def __iter__(self):
        return self

    def __next__(self) -> PCatVal:
        if self.index >= len(self._PCatVals):
            raise StopIteration
        idx = self.index
        self.index += 1
        return self._PCatVals[idx]



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.task import Task
from xerparser.model.classes.taskpred import TaskPred
from xerparser.model.classes.taskactv import TaskActv
from xerparser.model.predecessors import Predecessors
from typing import List


class Tasks:
    """
    This class is a collection of tasks that controls functionalities to search, add, update and delete tasks
    """
    

    def __init__(self):
        self.index = 0
        self._tasks = []

    def add(self, params, data):
        task = Task(params, data)
        self._tasks.append(task)

    @property
    def activities(self) -> List[Task]:
        return self._tasks

    @property
    def count(self):
        return len(self._tasks)

    @property
    def has_no_successor(self):
        objs = list(filter(lambda x: x.task_id not in [z.pred_task_id for z in TaskPred.obj_list], self._tasks))
        return objs

    @property
    def has_no_predecessor(self):
        objs = list(filter(lambda x: x.task_id not in [z.task_id for z in TaskPred.obj_list], self._tasks))
        return objs

    def __len__(self):
        return len(self._tasks)

    def __repr__(self):
        return [x.task_code for x in self._tasks]

    def __str__(self):
        return str([str(x.task_code) for x in self._tasks])

    @property
    def constraints(self):
        lst = [x.constraints if x.constraints is not None else None for x in self._tasks]
        # print(lst)
        return list(filter(lambda x: x is not None, lst))

    def find_by_id(self, id):
        obj = list(filter(lambda x: x.task_id == id, self._tasks))
        if len(obj) > 0:
            return obj[0]
        return obj

    def find_by_code(self, code):
        obj = list(filter(lambda x: x.task_code == code, self._tasks))
        if len(obj) > 0:
            return obj[0]
        return obj

    def duration_greater_than(self, duration):
        obj = list(filter(lambda x: x.target_drtn_hr_cnt > duration * float(self.calendar.day_hr_cnt), self._tasks))
        if obj:
            return obj
        return obj

    def float_less_than(self, Tfloat):
        objs = list(filter(lambda x: x.status_code != "TK_Complete", self._tasks))
        obj = list(filter(lambda x: x.total_float_hr_cnt < Tfloat * float(x.calendar.day_hr_cnt), objs))
        if obj:
            return obj
        return obj

    def float_greater_than(self, Tfloat):
        objs = list(filter(lambda x: x.status_code != "TK_Complete", self._tasks))
        obj = list(filter(lambda x: x.total_float_hr_cnt > Tfloat * float(x.calendar.day_hr_cnt), objs))
        if obj:
            return obj
        return obj

    def float_within_range(self, float1, float2):
        obj = None
        objs = list(filter(lambda x: x.status_code != "TK_Complete", self._tasks))
        if float1 < float2:
            obj = list(filter(lambda x: x.total_float_hr_cnt >= float1 * float(x.calendar.day_hr_cnt) and x.total_float_hr_cnt <= float2 * float(x.calendar.day_hr_cnt), objs))
            if obj:
                return obj
        return obj

    def float_within_range_exclusive(self, float1, float2):
        obj = None
        objs = list(filter(lambda x: x.status_code != "TK_Complete", self._tasks))
        if float1 < float2:
            obj = list(filter(lambda x: x.total_float_hr_cnt > float1 * float(x.calendar.day_hr_cnt) and x.total_float_hr_cnt < float2 * float(x.calendar.day_hr_cnt), objs))
            if obj:
                return obj
        return obj

    def activities_by_status(self, status):
        objs = list(filter(lambda x: x.status_code == status, self._tasks))
        return objs

    def activities_by_wbs_id(self, id):
        objs = list(filter(lambda x: x.wbs_id == id, self._tasks))
        return objs

    def activities_by_activity_code_id(self, id):
        objs = list(filter(lambda x: x.actv_code_id == id, TaskActv.obj_list))
        activities = []
        for obj in objs:
            activities.append(self.find_by_id(obj.task_id))
        return activities

    def no_predecessors(self):
        objs = list(filter(lambda x: x.task_id not in [z.task_id for z in TaskPred.obj_list], self._tasks))
        return objs

    def no_successors(self):
        objs = list(filter(lambda x: x.task_id not in [z.pred_task_id for z in TaskPred.obj_list], self._tasks))
        return objs

    def activities_with_hard_contratints(self):
        obj = list(filter(lambda x: x.cstr_type == "CS_MEO" or x.cstr_type == "CS_MSO", self._tasks))
        return obj

    def activities_by_type(self, type):
        obj = list(filter(lambda x: x.cstr_type == type, self._tasks))
        return obj
    
    def get_tsv(self):
        tsv = []
        if len(self._tasks) > 0:
            tsv.append(['%T', 'TASK'])
            tsv.append(['%F', 'task_id', 'proj_id', 'wbs_id', 'clndr_id', 'phys_complete_pct', 'rev_fdbk_flag',
               'est_wt', 'lock_plan_flag', 'auto_compute_act_flag', 'complete_pct_type', 'task_type',
               'duration_type', 'status_code', 'task_code', 'task_name', 'rsrc_id',
               'total_float_hr_cnt', 'free_float_hr_cnt', 'remain_drtn_hr_cnt', 'act_work_qty',
               'remain_work_qty', 'target_work_qty', 'target_drtn_hr_cnt', 'target_equip_qty',
               'act_equip_qty', 'remain_equip_qty', 'cstr_date', 'act_start_date', 'act_end_date',
               'late_start_date', 'late_end_date', 'expect_end_date', 'early_start_date',
               'early_end_date', 'restart_date', 'reend_date', 'target_start_date', 'target_end_date',
               'rem_late_start_date', 'rem_late_end_date', 'cstr_type', 'priority_type', 'suspend_date',
               'resume_date', 'float_path', 'float_path_order', 'guid', 'tmpl_guid', 'cstr_date2',
               'cstr_type2', 'driving_path_flag', 'act_this_per_work_qty', 'act_this_per_equip_qty',
               'external_early_start_date', 'external_late_end_date', 'create_date', 'update_date',
               'create_user', 'update_user', 'location_id'])
            for task in self._tasks:
                tsv.append(task.get_tsv())
        return tsv

    def get_by_project(self, id):
        obj = list(filter(lambda x: x.proj_id == id, self._tasks))
        return obj

    def __iter__(self):
        return self

    def __next__(self) -> Task:
        if self.index >= len(self._tasks):
            raise StopIteration
        idx = self.index
        self.index +=1
        return self._tasks[idx]



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.

import locale

class RoleRate:
    obj_list = []

    def __init__(self, params):
        self.role_rate_id = int(params.get('role_rate_id').strip()) if params.get('role_rate_id') else None
        self.role_id = int(params.get('role_id').strip()) if params.get('role_id') else None
        self.cost_per_qty = locale.atof(params.get('cost_per_qty').strip()) if params.get('cost_per_qty') else None
        self.cost_per_qty2 = locale.atof(params.get('cost_per_qty2').strip())if params.get('cost_per_qty2') else None
        self.cost_per_qty3 = locale.atof(params.get('cost_per_qty3').strip()) if params.get('cost_per_qty3') else None
        self.cost_per_qty4 = locale.atof(params.get('cost_per_qty4').strip()) if params.get('cost_per_qty4') else None
        self.cost_per_qty5 = locale.atof(params.get('cost_per_qty5').strip()) if params.get('cost_per_qty5') else None

        RoleRate.obj_list.append(self)

    @classmethod
    def find_by_id(cls, id):
        obj = list(filter(lambda x: x.role_rate_id == id, cls.obj_list))[0]
        return obj

    def get_tsv(self):
        tsv = ['%R', self.role_rate_id, self.role_id, self.cost_per_qty, self.cost_per_qty2,
               self.cost_per_qty3, self.cost_per_qty4, self.cost_per_qty5]
        return tsv

    @classmethod
    def find_by_role_id(cls, id):
        obj = list(filter(lambda x: x.role_id == id, cls.obj_list))
        if len(obj) > 0:
            obj = obj[0]
        else:
            obj = None
        return obj

    def __repr__(self):
        return str(self.role_rate_id)



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.taskrsrc import TaskRsrc
from xerparser.model.classes.rsrc import Resource


class ActivityResources:

    def __init__(self):
        self.index = 0
        self._taskrsrc = []

    def add(self, params, data):
        self._taskrsrc.append(TaskRsrc(params, data))

    def find_by_id(self, id) -> TaskRsrc:
        obj = list(filter(lambda x: x.taskrsrc_id == id, self._taskrsrc))
        if len(obj) > 0:
            return obj[0]
        return None
    
    def get_tsv(self):
        if len(self._taskrsrc) > 0:
            tsv = []
            tsv.append(['%T', 'TASKRSRC'])
            tsv.append(['%F', 'taskrsrc_id', 'task_id', 'proj_id', 'cost_qty_link_flag', 'role_id', 'acct_id',
               'rsrc_id', 'pobs_id', 'skill_level', 'remain_qty', 'target_qty', 'remain_qty_per_hr',
               'target_lag_drtn_hr_cnt', 'target_qty_per_hr', 'act_ot_qty', 'act_reg_qty',
               'relag_drtn_hr_cnt', 'ot_factor', 'cost_per_qty', 'target_cost', 'act_reg_cost',
               'act_ot_cost', 'remain_cost', 'act_start_date', 'act_end_date', 'restart_date',
               'reend_date', 'target_start_date', 'target_end_date', 'rem_late_start_date',
               'rem_late_end_date', 'rollup_dates_flag', 'target_crv', 'remain_crv', 'actual_crv',
               'ts_pend_act_end_flag', 'guid', 'rate_type', 'act_this_per_cost', 'act_this_per_qty',
               'curv_id', 'rsrc_type', 'cost_per_qty_source_type', 'create_user', 'create_date', 'cbs_id',
               'has_rsrchours', 'taskrsrc_sum_id'])
            for taskrsrc in self._taskrsrc:
                tsv.append(taskrsrc.get_tsv())
            return tsv
        return []

    def find_by_rsrc_id(self, id) -> TaskRsrc:
        obj = list(filter(lambda x: x.rsrc_id == id, self._taskrsrc))
        return obj

    def find_by_activity_id(self, id):
        obj = list(filter(lambda x: x.task_id == id and x.rsrc_id, self._taskrsrc))
        return obj

    @property
    def count(self):
        return len(self._taskrsrc)

    def __len__(self):
        return len(ActivityResources._taskrsrc)

    def __iter__(self):
        return self

    def __next__(self) -> TaskRsrc:
        if self.index >= len(self._taskrsrc):
            raise StopIteration
        idx = self.index
        self.index += 1
        return self._taskrsrc[idx]


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


class Role:
    obj_list = []

    def __init__(self, params):
        self.role_id = int(params.get('role_id')) if params.get('role_id') else None
        self.parent_role_id = int(params.get('parent_role_id')) if params.get('parent_role_id') else None
        self.seq_num = int(params.get('seq_num')) if params.get('seq_num') else None
        self.role_name = params.get('role_name') if params.get('role_name') else None
        self.role_short_name = params.get('role_short_name') if params.get('role_short_name') else None
        self.pobs_id = params.get('pobs_id') if params.get('pobs_id') else None
        self.def_cost_qty_link_flag = params.get('def_cost_qty_link_flag') if params.get('def_cost_qty_link_flag') else None
        self.cost_qty_type = params.get('cost_qty_type') if params.get('cost_qty_type') else None
        self.role_descr = params.get('role_descr') if params.get('role_descr') else None
        self.last_checksum = params.get('role_descr') if params.get('role_descr') else None

        Role.obj_list.append(self)

    def get_tsv(self):
        tsv = ['%R', self.role_id, self.parent_role_id, self.seq_num, self.role_name,
               self.role_short_name, self.pobs_id, self.def_cost_qty_link_flag, self.cost_qty_type,
               self.role_descr, self.last_checksum]
        return tsv

    def __repr__(self):
        return self.role_name



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.wbs import WBS
from typing import List


class WBSs:

    def __init__(self, data=None):
        self.index = 0
        self._wbss = []
        self.data = data

    def add(self, params, data):
        wbs = WBS(params, data)
        self._wbss.append(wbs)
    
    def get_tsv(self):
        tsv = []
        if len(self._wbss) > 0:
            tsv.append(['%T', 'PROJWBS'])
            tsv.append(['%F', 'wbs_id', 'proj_id', 'obs_id', 'seq_num', 'est_wt',
               'proj_node_flag', 'sum_data_flag', 'status_code', 'wbs_short_name',
               'wbs_name', 'phase_id', 'parent_wbs_id', 'ev_user_pct', 'ev_etc_user_value',
               'orig_cost', 'indep_remain_total_cost', 'ann_dscnt_rate_pct', 'dscnt_period_type',
               'indep_remain_work_qty', 'anticip_start_date', 'anticip_end_date', 'ev_compute_type',
               'ev_etc_compute_type', 'guid', 'tmpl_guid', 'plan_open_state'])
            for wb in self._wbss:
                tsv.append(wb.get_tsv())
        return tsv
    
    def get_by_project(self, id) -> List[WBS]:
        return list(filter(lambda x: x.proj_id == id, self._wbss))

    def __iter__(self):
        return self

    def __next__(self) -> WBS:
        if self.index >= len(self._wbss):
            raise StopIteration
        idx = self.index
        self.index += 1
        return self._wbss[idx]



# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.classes.project import Project
from xerparser.model.classes.data import Data

class Projects:


    def __init__(self):
        self.index =0
        self._projects = []

    def add(self, params, data):
        prj = Project(params, data)
        self._projects.append(prj)

    def get_tsv(self):
        if len(self._projects):
            tsv = []
            tsv.append(['%T', 'PROJECT'])
            tsv.append(['%F', 'proj_id', 'fy_start_month_num', 'rsrc_self_add_flag',
                   'allow_complete_flag', 'rsrc_multi_assign_flag', 'checkout_flag',
                   'project_flag', 'step_complete_flag', 'cost_qty_recalc_flag',
                   'batch_sum_flag', 'name_sep_char', 'def_complete_pct_type', 'proj_short_name',
                   'acct_id', 'orig_proj_id', 'source_proj_id', 'base_type_id', 'clndr_id',
                   'sum_base_proj_id', 'task_code_base', 'task_code_step', 'priority_num',
                   'wbs_max_sum_level', 'strgy_priority_num', 'last_checksum', 'critical_drtn_hr_cnt',
                   'def_cost_per_qty', 'last_recalc_date', 'plan_start_date', 'plan_end_date',
                   'scd_end_date', 'add_date', 'last_tasksum_date', 'fcst_start_date', 'def_duration_type',
                   'task_code_prefix', 'guid', 'def_qty_type', 'add_by_name', 'web_local_root_path',
                   'proj_url', 'def_rate_type', 'add_act_remain_flag', 'act_this_per_link_flag',
                   'def_task_type', 'act_pct_link_flag', 'critical_path_type', 'task_code_prefix_flag',
                   'def_rollup_dates_flag', 'use_project_baseline_flag', 'rem_target_link_flag',
                   'reset_planned_flag', 'allow_neg_act_flag', 'sum_assign_level', 'last_fin_dates_id',
                   'last_baseline_update_date', 'cr_external_key', 'apply_actuals_date', 'fintmpl_id', 'location_id',
                   'loaded_scope_level', 'export_flag', 'new_fin_dates_id', 'baselines_to_export',
                   'baseline_names_to_export', 'next_data_date', 'close_period_flag', 'sum_refresh_date',
                   'trsrcsum_loaded', 'sumtask_loaded'])
            for prj in self._projects:
                tsv.append(prj.get_tsv())
            return tsv
        return []

    def find_by_id(self, id) -> Project:
        obj = list(filter(lambda x: x.proj_id == id, self._projects))
        if obj:
            return obj[0]
        return obj

    def __repr__(self):
        return str(self._projects)

    def __iter__(self):
        return self

    def __next__(self) -> Project:
        if self.index >= len(self._projects):
            raise StopIteration
        idx = self.index
        self.index +=1
        return self._projects[idx]


# PyP6XER
# Copyright (C) 2020, 2021 Hassan Emam <hassan@constology.com>
#
# This file is part of PyP6XER.
#
# PyP6XER library is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License v2.1 as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# PyP6XER is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with PyP6XER.  If not, see <https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html>.


from xerparser.model.resources import Resources
import locale
class TaskRsrc:

    def __init__(self, params, data=None):
        self.taskrsrc_id = int(params.get('taskrsrc_id').strip()) if params.get('taskrsrc_id') else None
        self.task_id = int(params.get('task_id').strip()) if params.get('task_id') else None
        self.proj_id = params.get('proj_id').strip() if params.get('proj_id') else None
        self.cost_qty_link_flag = params.get('cost_qty_link_flag').strip() if params.get('cost_qty_link_flag') else None
        self.role_id = params.get('role_id').strip() if params.get('role_id') else None
        self.acct_id = params.get('acct_id').strip() if params.get('acct_id') else None
        self.rsrc_id = int(params.get('rsrc_id').strip()) if params.get('rsrc_id') else None
        self.pobs_id = params.get('pobs_id').strip() if params.get('pobs_id') else None
        self.skill_level = params.get('skill_level').strip() if params.get('skill_level') else None
        self.remain_qty = locale.atof(params.get('remain_qty').strip()) if params.get('remain_qty') else None
        self.target_qty = locale.atof(params.get('target_qty').strip()) if params.get('target_qty') else None
        self.remain_qty_per_hr = locale.atof(params.get('remain_qty_per_hr').strip()) if params.get('remain_qty_per_hr') else None
        self.target_lag_drtn_hr_cnt = locale.atof(params.get('target_lag_drtn_hr_cnt').strip()) if params.get('target_lag_drtn_hr_cnt') else None
        self.target_qty_per_hr = params.get('target_qty_per_hr').strip() if params.get('target_qty_per_hr') else None
        self.act_ot_qty = params.get('act_ot_qty').strip() if params.get('act_ot_qty') else None
        self.act_reg_qty = params.get('act_reg_qty').strip() if params.get('act_reg_qty') else None
        self.relag_drtn_hr_cnt = params.get('relag_drtn_hr_cnt').strip() if params.get('relag_drtn_hr_cnt') else None
        self.ot_factor = params.get('ot_factor').strip() if params.get('ot_factor') else None
        self.cost_per_qty = params.get('cost_per_qty').strip() if params.get('cost_per_qty') else None
        self.target_cost = params.get('target_cost').strip() if params.get('target_cost') else None
        self.act_reg_cost = locale.atof(params.get('act_reg_cost').strip()) if params.get('act_reg_cost') else None
        self.act_ot_cost = params.get('act_ot_cost').strip() if params.get('act_ot_cost') else None
        self.remain_cost = params.get('remain_cost').strip() if params.get('remain_cost') else None
        self.act_start_date = params.get('act_start_date').strip() if params.get('act_start_date') else None
        self.act_end_date = params.get('act_end_date').strip() if params.get('act_end_date') else None
        self.restart_date = params.get('restart_date').strip() if params.get('restart_date') else None
        self.reend_date = params.get('reend_date').strip() if params.get('reend_date') else None
        self.target_start_date = params.get('target_start_date').strip() if params.get('target_start_date') else None
        self.target_end_date = params.get('target_end_date').strip() if params.get('target_end_date') else None
        self.rem_late_start_date = params.get('rem_late_start_date').strip() if params.get('rem_late_start_date') else None
        self.rem_late_end_date = params.get('rem_late_end_date').strip() if params.get('rem_late_end_date') else None
        self.rollup_dates_flag = params.get('rollup_dates_flag').strip() if params.get('rollup_dates_flag') else None
        self.target_crv = params.get('target_crv').strip() if params.get('target_crv') else None
        self.remain_crv = params.get('remain_crv').strip() if params.get('remain_crv') else None
        self.actual_crv = params.get('actual_crv').strip() if params.get('actual_crv') else None
        self.ts_pend_act_end_flag = params.get('ts_pend_act_end_flag').strip() if params.get('ts_pend_act_end_flag') else None
        self.guid = params.get('guid').strip() if params.get('guid') else None
        self.rate_type = params.get('rate_type').strip() if params.get('rate_type') else None
        self.act_this_per_cost = params.get('act_this_per_cost').strip() if params.get('act_this_per_cost') else None
        self.act_this_per_qty = params.get('act_this_per_cost').strip() if params.get('act_this_per_cost') else None
        self.curv_id = params.get('curv_id').strip() if params.get('curv_id') else None
        self.rsrc_type = params.get('rsrc_type').strip() if params.get('rsrc_type') else None
        self.cost_per_qty_source_type = params.get('cost_per_qty_source_type').strip() if params.get('cost_per_qty_source_type') else None
        self.create_user = params.get('create_user').strip() if params.get('create_user') else None
        self.create_date = params.get('create_date').strip() if params.get('create_date') else None
        self.cbs_id = params.get('cbs_id').strip() if params.get('cbs_id') else None
        self.has_rsrchours = params.get('has_rsrchours').strip() if params.get('has_rsrchours') else None
        self.taskrsrc_sum_id = params.get('taskrsrc_sum_id').strip() if params.get('taskrsrc_sum_id') else None
        self.data = data

    def get_id(self):
        return self.taskrsrc_id
    @property
    def resource(self):
        return self.data.resources.get_resource_by_id(self.rsrc_id)

    def get_tsv(self):
        tsv = ['%R', self.taskrsrc_id, self.task_id, self.proj_id, self.cost_qty_link_flag, self.role_id, self.acct_id,
               self.rsrc_id, self.pobs_id, self.skill_level, self.remain_qty, self.target_qty, self.remain_qty_per_hr,
               self.target_lag_drtn_hr_cnt, self.target_qty_per_hr, self.act_ot_qty, self.act_reg_qty,
               self.relag_drtn_hr_cnt, self.ot_factor, self.cost_per_qty, self.target_cost, self.act_reg_cost,
               self.act_ot_cost, self.remain_cost, self.act_start_date, self.act_end_date, self.restart_date,
               self.reend_date, self.target_start_date, self.target_end_date, self.rem_late_start_date,
               self.rem_late_end_date, self.rollup_dates_flag, self.target_crv, self.remain_crv, self.actual_crv,
               self.ts_pend_act_end_flag, self.guid, self.rate_type, self.act_this_per_cost, self.act_this_per_qty,
               self.curv_id, self.rsrc_type, self.cost_per_qty_source_type, self.create_user, self.create_date,
               self.cbs_id, self.has_rsrchours, self.taskrsrc_sum_id]
        return tsv

    def __repr__(self):
        return str(self.task_id) + '->' + str(self.rsrc_id) + ' = ' + str(self.target_qty)
